const axios = require('axios');
const yaml = require('yaml');
const logger = require('../utils/logger');
const ClashParser = require('./clashParser');
const { Node, Subscription } = require('../models');
const sequelize = require('../config/database');

class MultiSubscriptionManager {
  constructor() {
    this.updateInterval = parseInt(process.env.SUBSCRIPTION_UPDATE_INTERVAL) || 3600000; // 默认1小时
    this.updateTimer = null;
    this.isUpdating = false; // 添加更新锁，防止并发更新
    this.speedTestDebounceTimer = null; // 防抖定时器
  }

  /**
   * 获取所有活跃的订阅
   */
  async getActiveSubscriptions() {
    try {
      return await Subscription.findAll({
        where: { isActive: true },
        order: [['createdAt', 'ASC']]
      });
    } catch (error) {
      logger.error('获取活跃订阅失败', { error: error.message });
      return [];
    }
  }

  /**
   * 从订阅地址获取配置
   */
  async fetchSubscription(subscription) {
    if (!subscription.url) {
      throw new Error('订阅地址为空');
    }

    try {
      const response = await axios.get(subscription.url, {
        timeout: 30000,
        headers: {
          'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36'
        }
      });

      if (!response.data) {
        throw new Error('订阅地址返回空内容');
      }

      let config;
      let rawContent = response.data;

      // 1. 先尝试YAML解析
      let yamlParsed = false;
      try {
        config = yaml.parse(rawContent);
        if (config && typeof config === 'object' && Array.isArray(config.proxies)) {
          yamlParsed = true;
        } else {
          throw new Error('不是有效的YAML订阅');
        }
      } catch (error) {
        // 尝试Base64解码
        
        // 2. 不是YAML，尝试整体Base64解码
        let decoded;
        try {
          decoded = Buffer.from(rawContent, 'base64').toString('utf8');
          
          // 如果解码后包含ss://、vmess://、trojan://，说明是明文链接合集
          if (/ss:\/\/|vmess:\/\/|trojan:\/\//.test(decoded)) {
            config = this.convertShadowsocksToClash(decoded);
          } else {
            // 3. 如果整体解码后不是明文链接合集，尝试多行Base64（每行一个链接）
            const lines = rawContent.split('\n').filter(line => line.trim());
            logger.info('原始内容行数:', lines.length, { subscriptionId: subscription.id });
            
            let decodedLines = [];
            for (const line of lines) {
              try {
                const d = Buffer.from(line, 'base64').toString('utf8');
                if (/ss:\/\/|vmess:\/\/|trojan:\/\//.test(d)) {
                  decodedLines.push(d);
                  logger.debug('成功解码一行:', d.substring(0, 50) + '...', { subscriptionId: subscription.id });
                }
              } catch (e) { 
                logger.debug('跳过无效的Base64行', { subscriptionId: subscription.id });
                continue; 
              }
            }
            
            logger.info('成功解码的行数:', decodedLines.length, { subscriptionId: subscription.id });
            if (decodedLines.length > 0) {
              config = this.convertShadowsocksToClash(decodedLines.join('\n'));
            } else {
              throw new Error('无法解析订阅内容，既不是有效的YAML也不是有效的Base64编码');
            }
          }
        } catch (decodeError) {
          logger.error('Base64解码失败:', decodeError.message, { subscriptionId: subscription.id });
          throw new Error('无法解析订阅内容，既不是有效的YAML也不是有效的Base64编码');
        }
      }

      if (!config.proxies || !Array.isArray(config.proxies)) {
        logger.error('订阅配置中没有找到有效的代理节点', {
          hasProxies: !!config.proxies,
          isArray: Array.isArray(config.proxies),
          configKeys: Object.keys(config || {}),
          subscriptionId: subscription.id
        });
        throw new Error('订阅配置中没有找到有效的代理节点');
      }
      
      // 解析成功

      return config;
    } catch (error) {
      logger.error('获取订阅配置失败', { 
        error: error.message, 
        url: subscription.url,
        subscriptionId: subscription.id
      });
      throw error;
    }
  }

  /**
   * 更新单个订阅的节点列表 - 使用事务保护
   */
  async updateSubscriptionNodes(subscription) {
    // 使用事务来确保数据一致性
    const transaction = await sequelize.transaction();
    
    try {
      logger.info('开始更新订阅节点', { 
        subscriptionId: subscription.id,
        subscriptionName: subscription.name 
      });

      // 获取订阅配置
      const config = await this.fetchSubscription(subscription);
      
      // 解析节点 - 在解析阶段就去重
      const parser = new ClashParser();
      const newNodes = [];
      const parsedKeys = new Set(); // 用于在解析阶段去重
      
      for (const proxy of config.proxies) {
        const node = parser.parseProxy(proxy);
        if (node) {
          const key = `${node.name}-${node.server}-${node.port}`;
          
          // 检查是否已经解析过相同的节点
          if (parsedKeys.has(key)) {
            logger.debug(`跳过重复解析的节点: ${node.name} (${node.server}:${node.port})`, {
              subscriptionId: subscription.id
            });
            continue;
          }
          
          parsedKeys.add(key);
          newNodes.push(node);
        }
      }
      
      // 解析完成，去重后节点数量: ${newNodes.length}

      if (newNodes.length === 0) {
        logger.warn('订阅中没有找到有效的节点', { subscriptionId: subscription.id });
        await transaction.rollback();
        return { updated: 0, added: 0, removed: 0 };
      }

      // 在事务中获取现有节点
      const existingNodes = await Node.findAll({
        where: { 
          subscriptionId: subscription.id,
          isActive: true 
        },
        transaction
      });

      const existingNodeMap = new Map();
      existingNodes.forEach(node => {
        const key = `${node.name}-${node.server}-${node.port}`;
        existingNodeMap.set(key, node);
      });

      let added = 0;
      let updated = 0;
      let removed = 0;

      // 处理新节点 - 使用数据库级别的去重
      const processedKeys = new Set();
      
      for (const nodeData of newNodes) {
        const key = `${nodeData.name}-${nodeData.server}-${nodeData.port}`;
        
        // 检查是否已经处理过相同的节点
        if (processedKeys.has(key)) {
          logger.warn(`跳过重复节点: ${nodeData.name} (${nodeData.server}:${nodeData.port})`, {
            subscriptionId: subscription.id
          });
          continue;
        }
        
        processedKeys.add(key);
        const existingNode = existingNodeMap.get(key);

        if (existingNode) {
          // 更新现有节点
          await existingNode.update({
            ...nodeData,
            subscriptionId: subscription.id,
            updatedAt: new Date()
          }, { transaction });
          updated++;
          existingNodeMap.delete(key);
        } else {
          // 检查数据库中是否已存在相同的节点（防止并发创建）
          const existingDuplicate = await Node.findOne({
            where: {
              name: nodeData.name,
              server: nodeData.server,
              port: nodeData.port,
              subscriptionId: subscription.id
            },
            transaction
          });
          
          if (existingDuplicate) {
            logger.warn(`数据库中已存在相同节点，跳过创建: ${nodeData.name}`, {
              subscriptionId: subscription.id
            });
            continue;
          }
          
          // 创建新节点
          await Node.create({
            ...nodeData,
            subscriptionId: subscription.id,
            isActive: true,
            status: 'offline'
          }, { transaction });
          added++;
        }
      }

      // 标记不再存在的节点为非活跃
      for (const [key, node] of existingNodeMap) {
        await node.update({ isActive: false }, { transaction });
        removed++;
      }

      // 更新订阅的节点数量
      const actualNodeCount = processedKeys.size;
      await subscription.update({
        nodeCount: actualNodeCount,
        lastUpdateTime: new Date()
      }, { transaction });

      // 提交事务
      await transaction.commit();

      logger.info(`订阅更新: ${subscription.name} - 新增${added}个，更新${updated}个，移除${removed}个`);

      // 如果有新增或更新的节点，触发测速
      if (added > 0 || updated > 0) {
        // 延迟5秒后触发测速，确保数据库操作完成
        if (this.speedTestDebounceTimer) {
          clearTimeout(this.speedTestDebounceTimer);
        }
        this.speedTestDebounceTimer = setTimeout(() => {
          this.triggerSpeedTest();
          this.speedTestDebounceTimer = null;
        }, 5000);
      }

      return { updated, added, removed, actualNodeCount };
    } catch (error) {
      // 回滚事务
      await transaction.rollback();
      logger.error('更新订阅节点失败', { 
        error: error.message,
        subscriptionId: subscription.id 
      });
      throw error;
    }
  }

  /**
   * 更新所有订阅的节点 - 添加锁机制防止并发
   */
  async updateAllSubscriptions() {
    // 检查是否正在更新，如果是则跳过
    if (this.isUpdating) {
      logger.warn('订阅更新正在进行中，跳过本次更新');
      return [];
    }
    
    this.isUpdating = true;
    
    try {
      const subscriptions = await this.getActiveSubscriptions();
      logger.info(`更新 ${subscriptions.length} 个订阅`);

      const results = [];
      
      // 串行执行订阅更新，避免并发问题
      for (const subscription of subscriptions) {
        try {
          const result = await this.updateSubscriptionNodes(subscription);
          results.push({
            subscriptionId: subscription.id,
            subscriptionName: subscription.name,
            ...result
          });
          
          // 每个订阅更新后稍作延迟，避免过于频繁的数据库操作
          await new Promise(resolve => setTimeout(resolve, 2000));
          
        } catch (error) {
          logger.error('更新订阅失败', {
            subscriptionId: subscription.id,
            subscriptionName: subscription.name,
            error: error.message
          });
          results.push({
            subscriptionId: subscription.id,
            subscriptionName: subscription.name,
            error: error.message
          });
        }
      }

      // 检查是否有新增或更新的节点，如果有则触发测速
      const hasUpdates = results.some(result => 
        result.added > 0 || result.updated > 0
      );

      if (hasUpdates) {
        // 延迟5秒后触发测速，确保数据库操作完成
        if (this.speedTestDebounceTimer) {
          clearTimeout(this.speedTestDebounceTimer);
        }
        this.speedTestDebounceTimer = setTimeout(() => {
          this.triggerSpeedTest();
          this.speedTestDebounceTimer = null;
        }, 5000);
      }

      return results;
    } catch (error) {
      logger.error('更新所有订阅失败', { error: error.message });
      throw error;
    } finally {
      // 释放锁
      this.isUpdating = false;
    }
  }

  /**
   * 启动自动更新
   */
  startAutoUpdate() {
    logger.info('启动多订阅自动更新', { 
      interval: `${this.updateInterval / 1000}秒` 
    });

    // 立即执行一次更新
    this.updateAllSubscriptions().catch(error => {
      logger.error('初始多订阅更新失败', { error: error.message });
    });

    // 设置定时更新
    this.updateTimer = setInterval(() => {
      this.updateAllSubscriptions().catch(error => {
        logger.error('定时多订阅更新失败', { error: error.message });
      });
    }, this.updateInterval);
  }

  /**
   * 停止自动更新
   */
  stopAutoUpdate() {
    if (this.updateTimer) {
      clearInterval(this.updateTimer);
      this.updateTimer = null;
      logger.info('多订阅自动更新已停止');
    }
  }

  /**
   * 手动更新所有订阅
   */
  async manualUpdate() {
    try {
      const results = await this.updateAllSubscriptions();
      return {
        success: true,
        data: results
      };
    } catch (error) {
      return {
        success: false,
        error: error.message
      };
    }
  }

  /**
   * 手动更新单个订阅
   */
  async manualUpdateSubscription(subscriptionId) {
    try {
      const subscription = await Subscription.findByPk(subscriptionId);
      if (!subscription) {
        throw new Error('订阅不存在');
      }

      const result = await this.updateSubscriptionNodes(subscription);
      return {
        success: true,
        data: {
          subscriptionId: subscription.id,
          subscriptionName: subscription.name,
          ...result
        }
      };
    } catch (error) {
      return {
        success: false,
        error: error.message
      };
    }
  }

  /**
   * 将Shadowsocks链接转换为Clash格式
   */
  convertShadowsocksToClash(shadowsocksContent) {
    const lines = shadowsocksContent.split('\n').filter(line => line.trim());
    const proxies = [];
    const proxyKeys = new Set(); // 用于去重
    
          // 开始解析Shadowsocks内容
    
    let ssCount = 0;
    let vmessCount = 0;
    let trojanCount = 0;
    let errorCount = 0;
    let duplicateCount = 0;
    
    for (const line of lines) {
      if (line.startsWith('ss://')) {
        ssCount++;
        try {
          const proxy = this.parseShadowsocksUrl(line);
          if (proxy) {
            const key = `${proxy.name}-${proxy.server}-${proxy.port}`;
            
            // 检查是否重复
            if (proxyKeys.has(key)) {
              duplicateCount++;
              logger.debug(`跳过重复的Shadowsocks节点: ${proxy.name} (${proxy.server}:${proxy.port})`);
              continue;
            }
            
            proxyKeys.add(key);
            proxies.push(proxy);
          } else {
            errorCount++;
          }
        } catch (error) {
          errorCount++;
        }
      } else if (line.startsWith('vmess://')) {
        vmessCount++;
      } else if (line.startsWith('trojan://')) {
        trojanCount++;
      }
    }
    
          // 解析完成 - SS: ${ssCount}个, VMess: ${vmessCount}个, Trojan: ${trojanCount}个
    
    if (proxies.length === 0) {
      logger.warn('没有找到任何有效的代理节点');
    }
    
    return {
      proxies: proxies,
      proxyGroups: [
        {
          name: 'Proxy',
          type: 'select',
          proxies: proxies.map(p => p.name)
        }
      ]
    };
  }

  /**
   * 解析Shadowsocks URL
   */
  parseShadowsocksUrl(url) {
    try {
      // 移除ss://前缀
      const base64Part = url.substring(5);
      
      // 分离服务器信息和备注
      const parts = base64Part.split('#');
      const serverInfo = parts[0];
      const remark = parts[1] ? decodeURIComponent(parts[1]) : '';
      
      // 分离服务器地址和认证信息
      const atIndex = serverInfo.lastIndexOf('@');
      if (atIndex === -1) {
        logger.warn('Shadowsocks URL格式错误，缺少@符号', { url: url.substring(0, 50) + '...' });
        return null;
      }
      
      const authPart = serverInfo.substring(0, atIndex);
      const serverPart = serverInfo.substring(atIndex + 1);
      
      // 解析服务器地址和端口
      const colonIndex = serverPart.lastIndexOf(':');
      if (colonIndex === -1) {
        logger.warn('Shadowsocks URL格式错误，缺少端口号', { serverPart });
        return null;
      }
      
      const server = serverPart.substring(0, colonIndex);
      const port = parseInt(serverPart.substring(colonIndex + 1));
      
      if (isNaN(port) || port <= 0 || port > 65535) {
        logger.warn('Shadowsocks URL端口号无效', { port: serverPart.substring(colonIndex + 1) });
        return null;
      }
      
      // 解析认证信息
      const decodedAuth = Buffer.from(authPart, 'base64').toString('utf8');
      const colonIndex2 = decodedAuth.indexOf(':');
      if (colonIndex2 === -1) {
        logger.warn('Shadowsocks认证信息格式错误', { decodedAuth });
        return null;
      }
      
      const method = decodedAuth.substring(0, colonIndex2);
      const password = decodedAuth.substring(colonIndex2 + 1);
      
      // 清理服务器地址（移除可能的\r字符）
      const cleanServer = server.replace(/\r/g, '');
      const cleanRemark = remark.replace(/\r/g, '');
      
      return {
        name: cleanRemark || `${cleanServer}:${port}`,
        type: 'ss',
        server: cleanServer,
        port: port,
        method: method,
        password: password
      };
    } catch (error) {
      logger.warn('解析Shadowsocks URL失败', { url: url.substring(0, 50) + '...', error: error.message });
      return null;
    }
  }

  /**
   * 触发测速
   */
  triggerSpeedTest() {
    try {
      // 通过事件触发测速，避免直接依赖
      if (this.onSpeedTestTrigger) {
        this.onSpeedTestTrigger();
      } else {
        logger.info('测速触发器未设置，跳过自动测速');
      }
    } catch (error) {
      logger.error('触发测速失败', { error: error.message });
    }
  }

  /**
   * 设置测速触发器
   */
  setSpeedTestTrigger(trigger) {
    this.onSpeedTestTrigger = trigger;
  }

  /**
   * 获取订阅状态
   */
  async getStatus() {
    const subscriptions = await this.getActiveSubscriptions();
    return {
      activeSubscriptions: subscriptions.length,
      updateInterval: this.updateInterval,
      autoUpdateEnabled: !!this.updateTimer,
      subscriptions: subscriptions.map(sub => ({
        id: sub.id,
        name: sub.name,
        nodeCount: sub.nodeCount,
        lastUpdateTime: sub.lastUpdateTime
      }))
    };
  }
}

module.exports = MultiSubscriptionManager; 