const axios = require('axios');
const yaml = require('yaml');
const logger = require('../utils/logger');
const ClashParser = require('./clashParser');
const { Node } = require('../models');

class SubscriptionManager {
  constructor() {
    this.subscriptionUrl = process.env.CLASH_SUBSCRIPTION_URL || 'http://so.xfxssr.me/api/v1/client/subscribe?token=7854d59f38ac51700730b9e782c5160c';
    this.updateInterval = parseInt(process.env.SUBSCRIPTION_UPDATE_INTERVAL) || 3600000; // 默认1小时
  }

  /**
   * 从订阅地址获取配置
   */
  async fetchSubscription() {
    if (!this.subscriptionUrl) {
      throw new Error('未配置订阅地址');
    }

    try {
      const response = await axios.get(this.subscriptionUrl, {
        timeout: 30000,
        headers: {
          'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36'
        }
      });

      if (!response.data) {
        throw new Error('订阅地址返回空内容');
      }

      let config;
      let rawContent = response.data;

      // 1. 先尝试YAML解析
      let yamlParsed = false;
      try {
        config = yaml.parse(rawContent);
        if (config && typeof config === 'object' && Array.isArray(config.proxies)) {
          logger.info('成功解析为YAML格式');
          yamlParsed = true;
        } else {
          logger.info('YAML解析结果不是有效的订阅对象，进入Base64解码流程');
          throw new Error('不是有效的YAML订阅');
        }
      } catch (error) {
        if (!yamlParsed) logger.info('不是YAML格式，尝试Base64解码');
        // 2. 不是YAML，尝试整体Base64解码
        let decoded;
        try {
          decoded = Buffer.from(rawContent, 'base64').toString('utf8');
          logger.info('Base64解码成功，内容长度:', decoded.length);
          // 如果解码后包含ss://、vmess://、trojan://，说明是明文链接合集
          if (/ss:\/\/|vmess:\/\/|trojan:\/\//.test(decoded)) {
            logger.info('检测到代理链接，转换为Clash格式');
            config = this.convertShadowsocksToClash(decoded);
          } else {
            logger.info('整体解码后不是代理链接，尝试多行Base64解码');
            // 3. 如果整体解码后不是明文链接合集，尝试多行Base64（每行一个链接）
            const lines = rawContent.split('\n').filter(line => line.trim());
            logger.info('原始内容行数:', lines.length);
            let decodedLines = [];
            for (const line of lines) {
              try {
                const d = Buffer.from(line, 'base64').toString('utf8');
                if (/ss:\/\/|vmess:\/\/|trojan:\/\//.test(d)) {
                  decodedLines.push(d);
                  logger.debug('成功解码一行:', d.substring(0, 50) + '...');
                }
              } catch (e) { 
                logger.debug('跳过无效的Base64行');
                continue; 
              }
            }
            logger.info('成功解码的行数:', decodedLines.length);
            if (decodedLines.length > 0) {
              config = this.convertShadowsocksToClash(decodedLines.join('\n'));
            } else {
              throw new Error('无法解析订阅内容，既不是有效的YAML也不是有效的Base64编码');
            }
          }
        } catch (decodeError) {
          logger.error('Base64解码失败:', decodeError.message);
          throw new Error('无法解析订阅内容，既不是有效的YAML也不是有效的Base64编码');
        }
      }

      if (!config.proxies || !Array.isArray(config.proxies)) {
        logger.error('订阅配置中没有找到有效的代理节点', {
          hasProxies: !!config.proxies,
          isArray: Array.isArray(config.proxies),
          configKeys: Object.keys(config || {})
        });
        throw new Error('订阅配置中没有找到有效的代理节点');
      }
      
      logger.info('订阅配置解析成功', {
        proxyCount: config.proxies.length,
        proxyNames: config.proxies.slice(0, 3).map(p => p.name)
      });

      return config;
    } catch (error) {
      logger.error('获取订阅配置失败', {
        error: error.message,
        url: this.subscriptionUrl
      });
      throw error;
    }
  }

  /**
   * 更新节点列表
   */
  async updateNodes() {
    try {
      // 简化日志输出

      // 获取订阅配置
      const config = await this.fetchSubscription();
      
      // 解析节点
      const parser = new ClashParser();
      const newNodes = [];
      
      for (const proxy of config.proxies) {
        const node = parser.parseProxy(proxy);
        if (node) {
          newNodes.push(node);
        }
      }

      if (newNodes.length === 0) {
        logger.warn('订阅中没有找到有效的节点');
        return { updated: 0, added: 0, removed: 0 };
      }

      // 获取现有节点
      const existingNodes = await Node.findAll({
        where: { isActive: true }
      });

      const existingNodeMap = new Map();
      existingNodes.forEach(node => {
        const key = `${node.name}-${node.server}-${node.port}`;
        existingNodeMap.set(key, node);
      });

      let added = 0;
      let updated = 0;
      let removed = 0;

      // 处理新节点
      for (const nodeData of newNodes) {
        const key = `${nodeData.name}-${nodeData.server}-${nodeData.port}`;
        const existingNode = existingNodeMap.get(key);

        if (existingNode) {
          // 更新现有节点
          await existingNode.update({
            ...nodeData,
            updatedAt: new Date()
          });
          updated++;
          existingNodeMap.delete(key);
        } else {
          // 添加新节点
          await Node.create({
            ...nodeData,
            isActive: true,
            status: 'offline'
          });
          added++;
        }
      }

      // 标记不再存在的节点为非活跃
      for (const [key, node] of existingNodeMap) {
        await node.update({ isActive: false });
        removed++;
      }

      logger.info(`节点列表更新完成 - 新增${added}个，更新${updated}个，移除${removed}个`);

      return { updated: newNodes.length, added, updated, removed };
    } catch (error) {
      logger.error('更新节点列表失败', { error: error.message });
      throw error;
    }
  }

  /**
   * 启动自动更新
   */
  startAutoUpdate() {
    if (!this.subscriptionUrl) {
      logger.warn('未配置订阅地址，跳过自动更新');
      return;
    }

    logger.info('启动订阅自动更新', { 
      interval: `${this.updateInterval / 1000}秒` 
    });

    // 立即执行一次更新
    this.updateNodes().catch(error => {
      logger.error('初始订阅更新失败', { error: error.message });
    });

    // 设置定时更新
    this.updateTimer = setInterval(() => {
      this.updateNodes().catch(error => {
        logger.error('定时订阅更新失败', { error: error.message });
      });
    }, this.updateInterval);
  }

  /**
   * 停止自动更新
   */
  stopAutoUpdate() {
    if (this.updateTimer) {
      clearInterval(this.updateTimer);
      this.updateTimer = null;
      logger.info('订阅自动更新已停止');
    }
  }

  /**
   * 手动更新
   */
  async manualUpdate() {
    try {
      const result = await this.updateNodes();
      return {
        success: true,
        data: result
      };
    } catch (error) {
      return {
        success: false,
        error: error.message
      };
    }
  }

  /**
   * 将Shadowsocks链接转换为Clash格式
   */
  convertShadowsocksToClash(shadowsocksContent) {
    const lines = shadowsocksContent.split('\n').filter(line => line.trim());
    const proxies = [];
    
    logger.info(`开始解析Shadowsocks内容，共${lines.length}行`);
    
    let ssCount = 0;
    let vmessCount = 0;
    let trojanCount = 0;
    
    for (const line of lines) {
      if (line.startsWith('ss://')) {
        ssCount++;
        try {
          const proxy = this.parseShadowsocksUrl(line);
          if (proxy) {
            proxies.push(proxy);
            logger.debug('成功解析SS节点', { name: proxy.name, server: proxy.server, port: proxy.port });
          } else {
            logger.warn('解析SS链接返回null', { line: line.substring(0, 50) + '...' });
          }
        } catch (error) {
          logger.warn('解析SS链接失败', { line: line.substring(0, 50) + '...', error: error.message });
        }
      } else if (line.startsWith('vmess://')) {
        vmessCount++;
        logger.info('检测到VMess链接，暂不支持解析');
      } else if (line.startsWith('trojan://')) {
        trojanCount++;
        logger.info('检测到Trojan链接，暂不支持解析');
      }
    }
    
    logger.info(`解析完成 - SS: ${ssCount}个, VMess: ${vmessCount}个, Trojan: ${trojanCount}个, 成功解析: ${proxies.length}个`);
    
    if (proxies.length === 0) {
      logger.warn('没有找到任何有效的代理节点');
    }
    
    return {
      proxies: proxies,
      proxyGroups: [
        {
          name: 'Proxy',
          type: 'select',
          proxies: proxies.map(p => p.name)
        }
      ]
    };
  }

  /**
   * 解析Shadowsocks URL
   */
  parseShadowsocksUrl(url) {
    try {
      // 移除ss://前缀
      const base64Part = url.substring(5);
      
      // 分离服务器信息和备注
      const parts = base64Part.split('#');
      const serverInfo = parts[0];
      const remark = parts[1] ? decodeURIComponent(parts[1]) : '';
      
      // 分离服务器地址和认证信息
      const atIndex = serverInfo.lastIndexOf('@');
      if (atIndex === -1) {
        logger.warn('Shadowsocks URL格式错误，缺少@符号', { url: url.substring(0, 50) + '...' });
        return null;
      }
      
      const authPart = serverInfo.substring(0, atIndex);
      const serverPart = serverInfo.substring(atIndex + 1);
      
      // 解析服务器地址和端口
      const colonIndex = serverPart.lastIndexOf(':');
      if (colonIndex === -1) {
        logger.warn('Shadowsocks URL格式错误，缺少端口号', { serverPart });
        return null;
      }
      
      const server = serverPart.substring(0, colonIndex);
      const port = parseInt(serverPart.substring(colonIndex + 1));
      
      if (isNaN(port) || port <= 0 || port > 65535) {
        logger.warn('Shadowsocks URL端口号无效', { port: serverPart.substring(colonIndex + 1) });
        return null;
      }
      
      // 解析认证信息
      const decodedAuth = Buffer.from(authPart, 'base64').toString('utf8');
      const colonIndex2 = decodedAuth.indexOf(':');
      if (colonIndex2 === -1) {
        logger.warn('Shadowsocks认证信息格式错误', { decodedAuth });
        return null;
      }
      
      const method = decodedAuth.substring(0, colonIndex2);
      const password = decodedAuth.substring(colonIndex2 + 1);
      
      // 清理服务器地址（移除可能的\r字符）
      const cleanServer = server.replace(/\r/g, '');
      const cleanRemark = remark.replace(/\r/g, '');
      
      return {
        name: cleanRemark || `${cleanServer}:${port}`,
        type: 'ss',
        server: cleanServer,
        port: port,
        method: method,
        password: password
      };
    } catch (error) {
      logger.warn('解析Shadowsocks URL失败', { url: url.substring(0, 50) + '...', error: error.message });
      return null;
    }
  }

  /**
   * 获取订阅状态
   */
  getStatus() {
    return {
      subscriptionUrl: this.subscriptionUrl,
      updateInterval: this.updateInterval,
      autoUpdateEnabled: !!this.updateTimer,
      lastUpdate: this.lastUpdateTime
    };
  }
}

module.exports = SubscriptionManager; 