const sequelize = require('../config/database');
const { Node, TestResult, Notification } = require('../models');
const logger = require('../utils/logger');

async function migrate() {
  try {
    logger.info('开始数据库迁移');

    // 同步所有模型
    await sequelize.sync({ force: false, alter: true });
    
    logger.info('数据库迁移完成');

    // 检查是否需要创建索引
    await createIndexes();

    logger.info('数据库迁移和索引创建完成');
    process.exit(0);
  } catch (error) {
    logger.error('数据库迁移失败', { error: error.message });
    process.exit(1);
  }
}

async function createIndexes() {
  try {
    // 创建额外的索引以提高查询性能
    const queries = [
      // 测试结果索引
      'CREATE INDEX IF NOT EXISTS idx_test_results_node_time ON test_results(nodeId, testTime DESC)',
      'CREATE INDEX IF NOT EXISTS idx_test_results_success ON test_results(isSuccess, testTime DESC)',
      
      // 通知索引
      'CREATE INDEX IF NOT EXISTS idx_notifications_type_time ON notifications(type, createdAt DESC)',
      'CREATE INDEX IF NOT EXISTS idx_notifications_sent ON notifications(isSent, retryCount)',
      
      // 节点索引
      'CREATE INDEX IF NOT EXISTS idx_nodes_status_group ON nodes(status, group)',
      'CREATE INDEX IF NOT EXISTS idx_nodes_last_test ON nodes(lastTestTime DESC)'
    ];

    for (const query of queries) {
      try {
        await sequelize.query(query);
      } catch (error) {
        // 索引可能已存在，忽略错误
        logger.debug('索引创建跳过', { query, error: error.message });
      }
    }

    logger.info('索引创建完成');
  } catch (error) {
    logger.error('创建索引失败', { error: error.message });
  }
}

// 如果直接运行此文件
if (require.main === module) {
  migrate();
}

module.exports = { migrate, createIndexes }; 